//! UCI is a simple library to allow people to ignore the lower-level protocol needed to create chess engines.
//! The engine is fully generic. By specifying a valid reader and writing you can send the messages to STDIN
//! and STDOUT, per the standard - or to memory for testing.


// BIG TODO: Change pub struct members to private and use getters/setters. DO THIS FOR EVERYTHING AFTER YOU
// IMPLEMENT OPTION SENDING.
// ALSO CHANGE COMMANDS TO HAVE A CONSTANTS SUBMODULE LIKE OPTIONS

extern crate either;

pub mod commands;
pub mod options;
pub mod parser;

use std::io::{BufRead, Write};
use std::vec;

#[derive(Debug)]
pub struct Engine<'a, R, W> {
  pub name: &'a str,
  pub author: &'a str,
  pub reader: R,
  pub writer: W,
  pub engine_options: Vec<options::EngineOption>,
}

/// Notes to delete later:
///
///
///  A way to make this comfortable for users to use is this provides easy access to commands as well as a
///  loop and stuff they can use. This way they don't have to override anything - they just include this
///  in their code for their engine to make the communication portion easy.
///
///  Think of this in the manner of a game engine. They don't provide the loop and everything, but they do
///  provide convenience functions and stuff to make _building_ that loop easier.
///
///  In other words, the engine writer still needs to know how the standard works. They just don't need to
///  implement all the nasty details.
///  For example, a start up function calls the correct "boot up" code for the engine so they only have to
///  worry about calling "boot up" prior to their main engine loop.
///
///
///  TODO: THE NEXT THING - THE PARSER! (after putting in getters but _no_ setters and taking away pub fields
///        for all structs in the app if that is idiomatic)
///
///  Additionally this code should have a parser. You call it with the reader supplied and it waits for a command
///  and parses it into a tuple of <token, value>. For example it would parse `setoption` into an option constant
///  and the value to set it to. This should be super generic and parsing results should just be of a type
///  ParseResult. The tuple implements ParseResult which represents a tuple of a token and value.

impl<'a, R, W> Engine<'a, R, W>
where
  R: BufRead,
  W: Write,
{
  pub fn new(name: &'a str, author: &'a str,
             reader: R, writer: W, engine_options: Vec<options::EngineOption>) -> Engine<'a, R, W> {
    Engine { name, author, reader, writer, engine_options,
    }
  }

  /// Sends identification messages to the writer for the GUI to pick up
  pub fn identify(&mut self) {
    let name_id: String = format!("{} {} {}\n", commands::ID, commands::NAME, self.name);
    let author_id: String = format!("{} {} {}\n", commands::ID, commands::AUTHOR, self.author);

    // For these two writes we can panic - there's no possibility of recovery if the engine fails at this stage
    write!(&mut self.writer, "{}", name_id).expect("failed to send name identification to writer");
    write!(&mut self.writer, "{}", author_id).expect("failed to send author identification to writer");
  }

  /// Sends all available options in the options configuration and a final UCIOK meaning we are reading to go.
  pub fn send_available_options(&mut self) {
    for eo in &self.engine_options {
      write!(&mut self.writer, "{}", eo.to_string()).expect(&format!("failed to send `{}`", eo.to_string()));
    }

    // Again this command must complete before we can say the engine is connected, so panicking at this stage is ok
    write!(&mut self.writer, "{}", commands::UCIOK).expect("failed to send `uciok` command");
  }

  /// Sends `readyok` to the GUI
  pub fn ready(&mut self) {
    write!(&mut self.writer, "{}", commands::READYOK).expect("failed to send `readyok` command");
  }
}
